import pygame
import os
from .misc import *
from .consts import *
from .player import *
from .enemies import *


class NPC(pygame.sprite.Sprite):
    def __init__(self, pos, message, background_pos=(0, 0)):
        pygame.sprite.Sprite.__init__(self, NPC.containers)

        self.image = pygame.Surface(PLAYER_SHAPE)
        self.image.fill(NPC_COLOR)

        self.rect = self.image.get_rect()

        self.pos = list(pos)
        self.update_pos(background_pos)

        self.message = None
        self.messages = message
        self.keypress = False
        self.touching = False
        self.talking = False

    def update_pos(self, background_pos):
        self.rect.topleft = (
            self.pos[0] + background_pos[0],
            self.pos[1] + background_pos[1],
        )

    def target(self, player):
        self.touching = pygame.sprite.spritecollide(self, [player], False)
        if self.message and not self.message.success:
            self.message = None

        if self.talking and not self.message:
            self.message = Message(self.messages)
        if self.message and distance_to(self.pos, player.pos) > NPC_MAX_DIST:
            self.keypress = False
            self.message.kill()
            self.message = None

    def keys(self, keys):
        if not self.keypress and keys[pygame.K_c] and self.touching:
            self.talking = True
        self.talking = self.talking and keys[pygame.K_c]
        self.keypress = keys[pygame.K_c]

    def update(self, background_pos):
        self.update_pos(background_pos)

    def kill(self):
        if self.message:
            self.message.kill()
        super().kill()


class Message(pygame.sprite.Sprite):
    message = False
    def __init__(self, messages, name="Generic NPC"):
        pygame.sprite.Sprite.__init__(self, Message.containers)

        self.image = pygame.Surface(MESSAGE_SHAPE, pygame.SRCALPHA)
        self.image.fill(MESSAGE_COLOR)

        self.rect = self.image.get_rect()

        self.rect.bottom = SCREEN_SIZE[1]

        self.success = True
        if Message.message:
            self.success = False
            super().kill()
        Message.message = True

        self.messages = messages
        self.name = name

        self.i = 0
        self.draw()

        self.keydown = False

    def draw(self):
        self.image.fill(MESSAGE_COLOR)
        self.image.fill((255, 255, 255, 100), pygame.Rect((0, 0), (MESSAGE_SHAPE[0], 50)))
        text = message_font.render(self.name, True, (0, 0, 0))
        self.image.blit(text, (10, 10))
        text = message_font.render(self.messages[self.i], True, (255, 255, 255))
        self.image.blit(text, (10, 100))

    def kill(self):
        Message.message = False
        self.success = False
        super().kill()

    def keys(self, keys):
        if keys[pygame.K_z]:
            if not self.keydown:
                self.i += 1
                self.keydown = True
                if self.i >= len(self.messages):
                    self.kill()
                else:
                    self.draw()
        else:
            self.keydown = False


class Healthbar(pygame.sprite.Sprite):
    def __init__(self):
        pygame.sprite.Sprite.__init__(self, Healthbar.containers)

        self.image = pygame.Surface(HEALTHBAR_SHAPE, pygame.SRCALPHA)

        self.rect = self.image.get_rect()

    def target(self, player):
        self.image.fill(HEALTHBAR_COLOR)
        max_width = HEALTHBAR_SHAPE[0] - 2*HEALTHBAR_PADDING
        health = player.health / PLAYER_MAX_HEALTH
        width = max_width * health
        pygame.draw.rect(
            self.image,
            HEALTH_COLOR,
            pygame.Rect((HEALTHBAR_PADDING, HEALTHBAR_PADDING), (width, HEALTHBAR_SHAPE[1] - HEALTHBAR_PADDING*2)),
        )


class Wall(pygame.sprite.Sprite):
    def __init__(self, pos, size, background_pos=(0, 0)):
       pygame.sprite.Sprite.__init__(self, Wall.containers)

       self.image = pygame.Surface(size)
       self.image.fill(BLOCK_COLOR)

       self.rect = self.image.get_rect()

       self.pos = list(pos)
       self.update_pos(background_pos)

    def update_pos(self, background_pos):
        self.rect.topleft = (
            self.pos[0] + background_pos[0],
            self.pos[1] + background_pos[1],
        )

    def update(self, background_pos):
        self.update_pos(background_pos)


class Next_Level(pygame.sprite.Sprite):
    def __init__(self, pos, size, inc, background_pos=(0, 0)):
       pygame.sprite.Sprite.__init__(self, Next_Level.containers)

       self.image = pygame.Surface(size)
       self.image.fill((255, 0, 0))

       self.rect = self.image.get_rect()

       self.pos = list(pos)

       self.inc = inc
       self.change = False

    def update_pos(self, background_pos):
        self.rect.topleft = (
            self.pos[0] + background_pos[0],
            self.pos[1] + background_pos[1],
        )

    def update(self, background_pos):
        self.update_pos(background_pos)

    def target(self, player):
        if pygame.sprite.spritecollide(self, [player], False):
            self.change = True


class Background(pygame.sprite.Sprite):
    def __init__(self, image, display, background_offset=(0, 0)):
        pygame.sprite.Sprite.__init__(self, Background.containers)

        one_image = pygame.image.load(os.path.join('data/images', image))
        self.one_size = one_image.get_size()

        display_size = display.get_size()

        size = (
            round(display_size[0], self.one_size[0]) + self.one_size[0]*2,
            round(display_size[1], self.one_size[1]) + self.one_size[1]*2,
        )

        self.image = pygame.Surface(size)

        for x in range(0, size[0], self.one_size[0]):
            for y in range(0, size[1], self.one_size[1]):
                self.image.blit(one_image, (x, y))

        self.rect = self.image.get_rect()

        self.pos = [0, 0]
        self.background_offset = list(background_offset)

    def offset(self, background_offset):
        self.background_offset[0] += background_offset[0]
        self.background_offset[1] += background_offset[1]

    def update(self, background_pos):
        self.rect.topleft = (
            (self.background_offset[0] + background_pos[0]) / 2 % self.one_size[0] - self.one_size[0],
            (self.background_offset[1] + background_pos[1]) / 2 % self.one_size[1] - self.one_size[1],
        )


class Camera:
    def __init__(self, screen, player):
        self.rect = screen.get_rect()
        self.pos = -player.pos[0]+self.rect.width/2, -player.pos[1]+self.rect.height/2
        self.update(player)

    def update(self, player):
        new_pos = -player.pos[0]+self.rect.width/2, -player.pos[1]+self.rect.height/2
        # self.pos = (self.pos[0]*5 + new_pos[0])/6, (self.pos[1]*5 + self.new_pos[1])/6
        self.pos = new_pos
