import pygame
from .consts import *
from .misc import *
from functools import reduce

__all__ = ['Player']

class Player(pygame.sprite.Sprite):
    def __init__(self, pos, background_pos=(0, 0)):
        pygame.sprite.Sprite.__init__(self, Player.containers)

        self.image = pygame.Surface(PLAYER_SHAPE)
        self.image.fill(PLAYER_COLOR)

        self.rect = self.image.get_rect()

        self.vel = [0, 0]

        self.pos = list(pos)

        self.jump = False

        self.arrows = (False, False, False, False)

        self.last_background = 0, 0

        self.health = PLAYER_MAX_HEALTH

        self.collide()

        self.dead_level = 0

    def update(self, background_pos):
        self.last_background = background_pos
        self.rect.topleft = self.topleft

    @property
    def topleft(self):
        return (
            int(self.pos[0]) + self.last_background[0],
            int(self.pos[1]) + self.last_background[1],
        )

    def collide(self):
        self.sides = (
            pygame.Rect((self.topleft[0], self.topleft[1]+1), (1, self.rect.height-2)),
            pygame.Rect((self.topleft[0]+1, self.topleft[1]), (self.rect.width-2, 1)),
            pygame.Rect((self.topleft[0]+self.rect.width-1, self.topleft[1]+1), (1, self.rect.height-2)),
            pygame.Rect((self.topleft[0]+1, self.topleft[1]+self.rect.height-1), (self.rect.width-2, 1)),
        )

    def take_damage(self, damage_group):
        colliding = pygame.sprite.spritecollide(self, damage_group, False)
        for c in colliding:
            self.health -= c.damage
        self.health = max(self.health, 0)

    def keys(self, keys):
        self.arrows = (
            keys[pygame.K_LEFT],
            keys[pygame.K_UP],
            keys[pygame.K_RIGHT],
            keys[pygame.K_DOWN],
        )

    def move(self, collidable_group):
        colliding = pygame.sprite.spritecollideany(self, collidable_group, False)
        self.vel[1] += PLAYER_GRAVITY_ACCEL

        if self.arrows[1] and colliding and not self.jump:
            self.vel[1] = -PLAYER_JUMP_VEL
            self.jump = True
        if not self.arrows[1] and colliding:
            self.jump = False

        if self.arrows[0] and not self.arrows[2]:
            self.vel[0] -= PLAYER_MOVE_ACCEL
        elif self.arrows[2] and not self.arrows[0]:
            self.vel[0] += PLAYER_MOVE_ACCEL
        else:
            friction = PLAYER_H_FRICTION if colliding else PLAYER_H_AIR_FRICTION
            self.vel[0] /= friction

        self.vel[0] = lock(self.vel[0], -PLAYER_MAX_H_VEL, PLAYER_MAX_H_VEL)
        self.vel[1] = lock(self.vel[1], -PLAYER_MAX_V_VEL, PLAYER_MAX_V_VEL)

        if -PLAYER_MIN_SPEED <= self.vel[0] <= PLAYER_MIN_SPEED:
            self.vel[0] = 0

        # self.vel = [0, 0]
        # if self.arrows[0] and self.vel[0] == 0:
        #     self.vel[0] = -2
        # if self.arrows[1] and self.vel[1] == 0:
        #     self.vel[1] = -2
        # if self.arrows[2] and self.vel[0] == 0:
        #     self.vel[0] = 2
        # elif self.arrows[2]:
        #     self.vel[0] = 0
        # if self.arrows[3] and self.vel[1] == 0:
        #     self.vel[1] = 2
        # elif self.arrows[3]:
        #     self.vel[1] = 0

        # if self.arrows[1]:
        #     self.vel[1] = -4
        # elif self.arrows[3]:
        #     self.vel[1] = 4
        # else:
        #     self.vel[1] = 0

        self.pos = [self.pos[0] + self.vel[0], self.pos[1] + self.vel[1]]

        self.update(self.last_background)
        self.collide()

        colliding = pygame.sprite.spritecollide(self, collidable_group, False)
        collisions = []
        sides = []
        for c in colliding:
            collisions.append([self.sides[i].colliderect(c.rect) for i in range(4)])
            sides.append([self.sides[i].clip(c.rect) for i in range(4)])

        if colliding:
            collisions = [any(x) for x in zip(*collisions)]
            # print()
            # print()
            sides = [x for x in zip(*sides)]
            # print(sides)
            sides = [tuple(filter(lambda rect: rect.width != 0, x)) for x in sides]
            # print(sides)
            sides = [reduce(lambda a, b:a.union(b), x) if x else None for x in sides]
            # print(sides)
            sides = [
                sides[0].height if sides[0] else 0,
                sides[1].width if sides[1] else 0,
                sides[2].height if sides[2] else 0,
                sides[3].width if sides[3] else 0,
            ]
            # print(sides)

            # Collide corners
            max_w = self.rect.width - 2
            max_h = self.rect.h - 2
            if (sides[0] == max_h and sides[1] == max_w) and not (sides[2] == max_h or sides[3] == max_w):
                # topleft collides
                self.vel = [0, 0]
                self.pos[0] += sides[3] + 1
                self.pos[1] += sides[2] + 1
            elif (sides[1] == max_w and sides[2] == max_h) and not (sides[3] == max_w or sides[0] == max_h):
                # topright collides
                self.vel = [0, 0]
                self.pos[0] -= sides[3] + 1
                self.pos[1] += sides[0] + 1
            elif (sides[2] == max_h and sides[3] == max_w) and not (sides[0] == max_h or sides[1] == max_w):
                # bottomright collides
                self.vel = [0, 0]
                self.pos[0] -= sides[1] + 1
                self.pos[1] -= sides[0]
            elif (sides[3] == max_w and sides[0] == max_h) and not (sides[1] == max_w or sides[2] == max_h):
                # bottomleft collides
                self.vel = [0, 0]
                self.pos[0] += sides[1] + 1
                self.pos[1] -= sides[2]

            else:
                if sides[3] != sides[1]:
                    if collisions[3] and sides[3] > sides[1]:
                        # bottom
                        self.vel[1] = min(0, self.vel[1])
                        self.pos[1] -= min(sides[0] or sides[2], sides[2] or sides[0])
                    if collisions[1] and sides[1] > sides[3]:
                        # top
                        self.vel[1] = max(0, self.vel[1])
                        self.pos[1] += min(sides[0] or sides[2], sides[2] or sides[0]) + 1

                self.update(self.last_background)
                self.collide()
                if sides[3] == sides[1]:
                    if collisions[2] and sides[2] > sides[0]:
                        self.vel[0] = 0
                        self.pos[0] -= min(sides[1], sides[3]) + 1
                    if collisions[0] and sides[0] > sides[2]:
                        self.vel[0] = 0
                        self.pos[0] += min(sides[1], sides[3]) + 1
